FSW (Facebook SDK Wrapper)

First, let me note that neither me nor this code is affiliated with Facebook. That's probably obvious, but corporations can be weird about perceived encroachment on their brand, so I just wanted to get that out of the way. This is an independent work that leverages the Facebook JavaScript SDK, but is not in any sense a part of it.

FSW is a small JavaScript library the provides an alternative interface to the Facebook JavaScript SDK. Why would you want this? I have found working with the official SDK difficult because the documentation is often unclear or contradictory, the interface is inconsistent, and keeping track of breaking changes is a challenge. This wrapper is my answer to those shortcomings.

Here's what you get:

- 	Automatic page set up. The wrapper loads the official Facebook JavaScript SDK, then adds the #fb-root div to your page and adds an XML namespace to the &lt;html> root for rendering XFBML tags (such as the like button).

- 	Accurate documentation. The docs are generated directly from the code, so there should be no discrepancy between how the docs say to call a method and the actual way to call the method.

- 	Versioning. I'm using [semantic versioning](http://semver.org/), which is a promise that a particular release will support a particular interface. Since this is a wrapper around an API I don't control, I can't promise that a change on Facebook won't break a particular version of the wrapper, but I can promise that a given version's interface is stable, and that any changes to the code (to keep up with a Facebook change, or to add more features) will be clearly indicated, with breaking vs non-breaking changes clearly signaled.

Note that the wrapper does not override the FB object provided by the official SDK. This means that you are not forced to always implement and SDK call via the wrapper; you still have the same access to "FB" that you did if you were not using the wrapper. It also means that, for those SDK methods that the wrapper does not yet support, you can still make those calls on your own. This wrapper is "in addition to," not "in place of."

Ready? Check out the [docs](http://www.philipschweiger.com/fsw/docs/) and the [example application](http://www.philipschweiger.com/fsw/)</a>.

I welcome suggestions and, even better, contributions. Is there an SDK method not yet supported by the wrapper that you really need? Fork this project, or send me a pull request!

Philip Schweiger &lt;pschwei1 AT gmail>