    //Test Values
var appId = '220491094663866',
    canvasUrl = 'http://apps.facebook.com/yourapphere/',
    tabUrl = 'http://www.facebook.com/pages/pagename/pageid?sk='+appId;
	
fsw.ready(function(){
module("Initialization");

    test ("at runtime: xfbml added to html namespace",function(){
		var xfbml = document.documentElement.attributes.getNamedItem('xmlns:fb');
        ok(xfbml);
		equal(xfbml.nodeValue,'https://www.facebook.com/2008/fbml');
   });
   
    test ("at runtime: #fb-root added to page",function(){
		var fbroot = document.getElementById('fb-root');
			ok(fbroot);
    });

    test ("at runtime: External FB JS library loaded",function(){
        ok(FB);
    });

    test("fsw.init: fails on missing requirements", function() {

        raises(
            function(){fsw.init();},
            function(err) {
                return err.message === 'fsw.init(): Missing appId';
            },
            'Missing appId'
        );
    });
	
    test("fsw.init: sets optional values to null if they are not passed", function() {
		fsw.init(appId);
		ok(fsw.getCanvasUrl() === null);
        ok(fsw.getTabUrl() === null);        
    });

    test("fsw.init: sets basic application info",function(){
        var stub = sinon.stub(FB,'init');

        fsw.init(appId,canvasUrl,tabUrl);

        equal(fsw.getAppId(),appId);
        equal(fsw.getCanvasUrl(),canvasUrl);
        equal(fsw.getTabUrl(),tabUrl);
        ok(stub.calledWithExactly({appId:appId,status:true,cookie:true,xfbml:true,oauth:true}));

        FB.init.restore();
    });

module("Authorization");

    test('auth:createBtn: fails on missing requirements',function(){
		var testDiv = document.getElementById('qunit-fixture');
			trigger = 'fblog',
			triggerHtml = document.createElement("button");
		triggerHtml.id = trigger;
		testDiv.appendChild(triggerHtml);	
		
        raises(
            function(){
                fsw.auth.createBtn();
            },
            'missing trigger'
        );

        raises(
            function(){
                fsw.auth.createBtn('fblogin',function(){});
            },
            'No element with ID "fblogin" exists in DOM'
        );

        raises(
            function(){
                fsw.auth.createBtn(trigger);
            },
            'missing callback'
        );
    });

    test('auth.createBtn: calls FB.login, basic',function(){
	
		var stubFblogin = sinon.stub(FB,'login'),
			callback = sinon.spy(),
			trigger = document.createElement("button");
		
		trigger.id = 'fblogin';
		document.getElementById('qunit-fixture').appendChild(trigger);

        fsw.auth.createBtn('fblogin', callback);
        trigger.click();
        ok(stubFblogin.calledWith(callback,null));
    
        FB.login.restore();
		
	});

    test('auth.createBtn: calls FB.login, permissions',function(){

        var stubFblogin = sinon.stub(FB,'login'),
            callback = sinon.spy(),
            permissionsIn = 'user_about_me, user_activities',
            permissionsOut = {perms:'user_about_me, user_activities'},
			trigger = document.createElement("button");
		
		trigger.id = 'fblogin';
		document.getElementById('qunit-fixture').appendChild(trigger);
		
        //Permissions
        fsw.auth.createBtn('fblogin', callback, permissionsIn);
        trigger.click();
        ok(stubFblogin.calledWith(callback,permissionsOut));

        FB.login.restore();
   });

module("Like, and its discontents",{
	setup:function(){
		this.stubFBParse = sinon.stub(FB.XFBML,'parse');
	},
	teardown:function(){
		FB.XFBML.parse.restore();
	}
});

    test('like.createBtn: fails on missing requirements',function(){

		raises(
            function(){fsw.like.createBtn();},
            'Missing container'
        );

        raises(
            function(){fsw.like.createBtn('likeholderBad');},
            'No element with ID "likeholderBad" exists in DOM'
        );

        raises(
            function(){fsw.like.createBtn('likeholder');},
            'Missing "like" target'
        );

    });

    test('like.createBtn: Passing Options',function(){
		var fixture = document.getElementById('qunit-fixture'),
			container1 = document.createElement("div"),
			container2 = document.createElement("div"),
			
			//Array of attributes we expect to see in generated HTML
			testArray1 = {
				href:tabUrl,
				action:"like",
				send:"false",
				width:"450",
				show_faces:"false",
				font:"arial",
				layout:"standard",
				colorscheme:"light",
				ref:''
			},
			testArray2 = {
				href:tabUrl,
				action:"recommend",
				send:"true",
				width:"550",
				show_faces:"true",
				font:"tahoma",
				layout:"box_count",
				colorscheme:"dark",
				ref:"test2"
			};
		
		container1.id = 'likeholder1';
		container2.id = 'likeholder2';
	
		fixture.appendChild(container1);
		fixture.appendChild(container2);
	
        var likeBtn1 = fsw.like.createBtn(container1.id,tabUrl),
            likeBtn2 = fsw.like.createBtn(container2.id,tabUrl,{
                action:'recommend',
                send:true,
                width:'550',
                faces:true,
                font:'tahoma',
                layout:'box_count',
                colorscheme:'dark',
                ref:'test2'
            }),
			
			inArray = function (val, array){
				for ( var i = 0, length = array.length; i < length; i++ ) {
					if ( array[i] === val) {
						return true;
					}
				}
				return false;
			};

        var results1 = container1.children[0].attributes,
			results2 = container2.children[0].attributes;
		
		for (x in testArray1) {
			equal(results1[x].value,testArray1[x]);
		}
		for (x in testArray2) {
			equal(results2[x].value,testArray2[x]);
		}
    });

   test('like.bind: fails on missing requirements',function(){
		var fixture = document.getElementById('qunit-fixture'),
			container = document.createElement("div");
		container.id = 'likeholder';
		fixture.appendChild(container);
	
        var likeSamplePage = fsw.like.createBtn(container.id,tabUrl);

        raises(
            function(){likeSamplePage.bind()},
            'missing onLike callback'
        );
        raises(
            function(){likeSamplePage.bind('do something')},
            'onLike callback must be a function'
        );
        raises(
            function(){likeSamplePage.bind(function(){})},
            'missing onUnlike callback'
        );
        raises(
            function(){likeSamplePage.bind(function(){},'do something')},
            'onUnlike callback must be a function'
        );

    });

    test('like.bind: callbacks bound',function(){
		var fixture = document.getElementById('qunit-fixture'),
			container = document.createElement("div"),
            stubEvtSubscribe = sinon.spy(FB.Event,'subscribe'),
            onLike = sinon.spy(),
            onUnlike = sinon.spy();
			
		container.id = 'likeholder';
		fixture.appendChild(container);

		var likeSamplePage = fsw.like.createBtn(container.id,tabUrl);

        likeSamplePage.bind(onLike,onUnlike);
        ok(stubEvtSubscribe.calledWith('edge.create',onLike));
        ok(stubEvtSubscribe.calledWith('edge.remove',onUnlike));

        //cleanup
        FB.Event.subscribe.restore();
    });

module("Send Button",{
	setup:function(){
		this.stubFBParse = sinon.stub(FB.XFBML,'parse');
	},
	teardown:function(){
		FB.XFBML.parse.restore();
	}
});

    test('send.createBtn: fails on missing requirements',function(){

        raises(
            function(){fsw.send.createBtn();},
            'Missing container'
        );

        raises(
            function(){fsw.send.createBtn('sendholderBad');},
            'No element with ID "sendholderBad" exists in DOM'
        );

        raises(
            function(){fsw.send.createBtn('sendholder');},
            'Missing "send" target'
        );
		
    });

    test('send.createBtn: Passing Options',function(){
		var fixture = document.getElementById('qunit-fixture'),
			container1 = document.createElement("div"),
			container2 = document.createElement("div"),
			
			//Array of attributes we expect to see in generated HTML
			testArray1 = {
				href:tabUrl,
				font:"arial",
				colorscheme:"light",
				ref:''
			},
			testArray2 = {
				href:tabUrl,
				font:"tahoma",
				colorscheme:"dark",
				ref:"test2"
			};
			
		container1.id = 'sendholder1';
		container2.id = 'sendholder2';
		
		fixture.appendChild(container1);
		fixture.appendChild(container2);
		
	
        var sendBtn1 = fsw.send.createBtn(container1.id,tabUrl),
            sendBtn2 = fsw.send.createBtn(container2.id,tabUrl,{
                font:'tahoma',
                colorscheme:'dark',
                ref:'test2'
            });

			
		var results1 = container1.children[0].attributes,
			results2 = container2.children[0].attributes;
			
		for (x in testArray1) {
			equal(results1[x].value,testArray1[x]);
		}
		for (x in testArray2) {
			equal(results2[x].value,testArray2[x]);
		}

    });

    test('send.bind: fails on missing requirements',function(){
		var fixture = document.getElementById('qunit-fixture'),
			container = document.createElement("div");
		container.id = 'sendholder';
		fixture.appendChild(container);
	
        var sendSamplePage = fsw.like.createBtn(container.id,tabUrl);
	
        raises(
            function(){sendSamplePage.bind()},
            'missing onSend callback'
        );
        raises(
            function(){sendSamplePage.bind('do something')},
            'onSend callback must be a function'
        );
    });

    test('send.bind: callbacks bound',function(){
		var fixture = document.getElementById('qunit-fixture'),
			container = document.createElement("div"),
            stubEvtSubscribe = sinon.spy(FB.Event,'subscribe'),
            onSend = sinon.spy();
			
		container.id = 'sendholder';
		fixture.appendChild(container);

		var sendSamplePage = fsw.send.createBtn(container.id,tabUrl);
	
        sendSamplePage.bind(onSend);
        ok(stubEvtSubscribe.calledWith('message.send',onSend));

        //cleanup
        FB.Event.subscribe.restore();
    });

module("stream",{
    setup:function(){
        this.stubFBui = sinon.stub(FB,'ui');
    },
    teardown:function(){
        FB.ui.restore();
    }
});

    test('stream.post: callled with defaults',function(){
        fsw.stream.post();

        ok(this.stubFBui.calledWith({
            actions:null,
            caption: ' ',
            description: ' ',
            link:null,
            method: 'feed',
            name: ' ',
            source:null
            },null
        ));

    })

    test('stream.post: called with passed parameters',function(){
        var callback = sinon.spy(),
            title = 'Stream Post Test',
            subtitle = 'Making FB API easier',
            link = window.location.toString(),
            description = 'Description goes here.',
            pic = 'http://www.example.com/image.jpg',

            options = {
                title:title,
                subtitle:subtitle,
                link:link,
                description:description,
                media:pic,
                action:'click,http://www.example.com/'
            };

        fsw.stream.post(options,callback);

        ok(this.stubFBui.calledWith(
            {
                actions:{
                    name:'click',
                    link:'http://www.example.com/'
                },
                caption:subtitle,
                description:description,
                link:link,
                method:'feed',
                name:title,
                source:pic
            },
            callback
        ));

    });

    test('stream.post: with a swf attachment, no thumbnail',function(){
        var callback = sinon.spy(),
            title = 'Stream Post Test',
            subtitle = 'Making FB API easier',
            link = window.location.toString(),
            description = 'Description goes here.',
            swf = 'http://www.example.com/movie.swf',

            options = {
                title:title,
                subtitle:subtitle,
                link:link,
                description:description,
                media:swf,
                action:'click,http://www.example.com/'
            };


        raises(
            function(){
                fsw.stream.post(options,callback);
            }
        );

    });

    test('stream.post: with a swf attachment, has thumbnail',function(){
        var callback = sinon.spy(),
            title = 'Stream Post Test',
            subtitle = 'Making FB API easier',
            link = window.location.toString(),
            description = 'Description goes here.',
            swf = 'http://www.example.com/movie.swf, http://www.example.com/thumb.jpg',

            options = {
                title:title,
                subtitle:subtitle,
                link:link,
                description:description,
                media:swf,
                action:'click,http://www.example.com/'
            };

		fsw.stream.post(options,callback);

        ok(this.stubFBui.calledWith(
            {
                actions:{
                    name:'click',
                    link:'http://www.example.com/'
                },
                caption:subtitle,
                description:description,
                link:link,
                method:'feed',
                name:title,
                picture:'http://www.example.com/thumb.jpg',
                source:'http://www.example.com/movie.swf'
            },
            callback
        ));
    });

module("Requests",{
    setup:function(){
        this.stubFBui = sinon.stub(FB,'ui');
    },
    teardown:function(){
        FB.ui.restore();
    }
});

    test("request.send: fails on missing requirements",function(){

        raises(function(){fsw.request.send()});

    });

    test("request.send: called with passed parameter",function(){
        var callback = sinon.spy(),
            to ='fbid goes here',
            message = 'Message here',
            data = 'Additional data here';

        //Minimum
        fsw.request.send({message:message});
        ok(this.stubFBui.calledWith({
            data:null,
            message:message,
            method:'apprequests',
            to:null
        },null),'basic');

        //To all friends
        fsw.request.send({
            message:message,
            data:data
        },callback);

        ok(this.stubFBui.calledWith(
            {
                data:data,
                message:message,
                method:'apprequests',
                to:null
            },
            callback
        ),'To all friends');

        //To specific friend
        fsw.request.send({
            message:message,
            data:data,
            to:to
        },callback);

        ok(this.stubFBui.calledWith(
            {
                data:data,
                message:message,
                method:'apprequests',
                to:to
            },
            callback
        ),'To specific friend');

    });
	
});